from unittest import TestCase
from opensearch import Results, Query

class ResultsTests(TestCase):

    def test_result(self):
        query = Query("http://www.koders.com/?s={searchTerms}&amp;p={startPage}&amp;output=rss&c={count}&amp;i={startIndex}")
        query.searchTerms = "python"
        results = Results(query)
        self.assertEqual(True, (results.totalResults > 0))
        self.assertEqual(results.startIndex, 1)
        self.assertEqual(results.itemsPerPage, 25)
        self.assertEqual(True, len(results.items) > 0)

    def test_iterator_with_index(self):
        query = Query("http://www.koders.com/?s={searchTerms}&amp;p={startPage}&amp;output=rss&c={count}&amp;i={startIndex}")
        query.searchTerms = "python"
        results = Results(query)
        count = 0 
        for i in results:
            count += 1
            if count > 75: 
                break
        self.assertEqual(True, count > 75)


    def test_iterator_with_page(self):
        query = Query("http://www.koders.com/?s={searchTerms}&amp;p={startPage}&amp;output=rss")
        query.searchTerms = "python"
        results = Results(query)
        count = 0 
        for i in results:
            count += 1
            if count > 75: 
                break
        self.assertEqual(True, count > 75)




from urlparse import urlparse, urlunparse
from urllib import urlencode
from cgi import parse_qs

class Query:
    """Represents an opensearch query. Used internally by the Client to 
    construct an opensearch url to request. Really this class is just a 
    helper for substituting values into the macros in a format. 

    format = 'http://beta.indeed.com/opensearch?q={searchTerms}&start={startIndex}&limit={count}'
    q = Query(format)
    q.searchTerms('zx81')
    q.startIndex = 1
    q.count = 25
    print q.to_url()
    """

    standard_macros = ['searchTerms','count','startIndex','startPage', 
        'language', 'outputEncoding', 'inputEncoding']

    def __init__(self, format):
        """Create a query object by passing it the url format obtained
        from the opensearch Description.
        """
        self.format = format

        # unpack the url to a tuple
        self.url_parts = urlparse(format)

        # unpack the query string to a dictionary
        self.query_string = parse_qs(self.url_parts[4])

        # look for standard macros and create a mapping of the 
        # opensearch names to the service specific ones
        # so q={searchTerms} will result in a mapping between searchTerms and q
        self.macro_map = {}
        for key,values in self.query_string.items():
            # TODO eventually optional/required params should be 
            # distinguished somehow (the ones with/without trailing ?
            macro = values[0].replace('{','').replace('}','').replace('?','')
            if macro in Query.standard_macros:
                self.macro_map[macro] = key

    def url(self):
        # copy the original query string
        query_string = dict(self.query_string)

        # iterate through macros and set the position in the querystring
        for macro, name in self.macro_map.items():
            if hasattr(self, macro):
                # set the name/value pair
                query_string[name] = [getattr(self, macro)]
            else:
                # remove the name/value pair
                del(query_string[name])

        # copy the url parts and substitute in our new query string
        url_parts = list(self.url_parts)
        url_parts[4] = urlencode(query_string, 1)

        # recompose and return url
        return urlunparse(tuple(url_parts))

    def has_macro(self, macro):
        return self.macro_map.has_key(macro)




from unittest import TestCase
from opensearch import Query

class QueryTests(TestCase):

    template = "http://www.koders.com/?s={searchTerms}&amp;p={startPage}&amp;output=rss&c={count}&amp;i={startIndex}"

    def test_searchTerms(self):
        q = Query(QueryTests.template)
        q.searchTerms = 'zx81'
        self.assertEqual(q.url(), 'http://www.koders.com/?s=zx81&output=rss')

    def test_count(self):
        q = Query(QueryTests.template)
        q.count = 25
        self.assertEqual(q.url(), 'http://www.koders.com/?c=25&output=rss')

    def test_startPage(self):
        q = Query(QueryTests.template)
        q.startPage = 5
        self.assertEqual(q.url(), 'http://www.koders.com/?p=5&output=rss')

    def test_startIndex(self):
        q = Query(QueryTests.template)
        q.startIndex = 5
        self.assertEqual(q.url(), 'http://www.koders.com/?i=5&output=rss')

    def test_multiple(self):
        q = Query(QueryTests.template)
        q.searchTerms = 'zx81'
        q.startIndex = 15
        q.count = 15
        self.assertEqual(q.url(), 
            'http://www.koders.com/?i=15&s=zx81&c=15&output=rss')




from description import Description
from query import Query
from results import Results

class Client:

    """This is the class you'll probably want to be using. You simply
    pass the constructor the url for the service description file and
    issue a search and get back results as an iterable Results object.

    The neat thing about a Results object is that it will seamlessly
    handle fetching more results from the opensearch server when it can...
    so you just need to iterate and can let the paging be taken care of 
    for you.

    from opensearch import Client
    client = Client(description_url)
    results = client.search("computer")
    for result in results:
        print result.title
    """

    def __init__(self, url, agent="python-opensearch <https://github.com/edsu/opensearch>"):
        self.agent = agent
        self.description = Description(url, self.agent)

    def search(self, search_terms, page_size=25):
        """Perform a search and get back a results object
        """
        url = self.description.get_best_template()
        query = Query(url)

        # set up initial values
        query.searchTerms = search_terms
        query.count = page_size

        # run the results
        return Results(query, agent=self.agent)




from opensearch import osfeedparser

class Results(object):

    def __init__(self, query, agent=None):
        self.agent = agent
        self._fetch(query)
        self._iter = 0

    def __iter__(self):
        self._iter = 0
        return self

    def __len__(self):
      return self.totalResults

    def next(self):

        # just keep going like the energizer bunny
        while True:

            # return any item we haven't returned
            if self._iter < len(self.items):
                self._iter += 1
                return self.items[self._iter-1]
          
            # if there appears to be more to fetch
            if \
                self.totalResults != 0 \
                and self.totalResults > self.startIndex + self.itemsPerPage - 1:

                # get the next query
                next_query = self._get_next_query()

                # if we got one executed it and go back to the beginning
                if next_query:
                    self._fetch(next_query)
                    # very important to reset this counter 
                    # or else the return will fail
                    self._iter = 0

            else:
                raise StopIteration


    def _fetch(self, query):
        feed  = osfeedparser.opensearch_parse(query.url(), agent=self.agent)
        self.feed = feed

        # general channel stuff
        channel = feed['feed']
        self.title = _pick(channel,'title')
        self.link = _pick(channel,'link')
        self.description = _pick(channel,'description')
        self.language = _pick(channel,'language')
        self.copyright = _pick(channel,'copyright')

        # get back opensearch specific values
        self.totalResults = _pick(channel,'opensearch_totalresults',0)
        self.startIndex = _pick(channel,'opensearch_startindex',1) 
        self.itemsPerPage = _pick(channel,'opensearch_itemsperpage',0)

        # alias items from the feed to our results object
        self.items = feed['items']

        # set default values if necessary
        if self.startIndex == 0:
            self.startIndex = 1
        if self.itemsPerPage == 0 and len(self.items) > 0:
            self.itemsPerPage = len(self.items)

        # store away query for calculating next results
        # if necessary
        self.last_query = query


    def _get_next_query(self):
        # update our query to get the next set of records
        query = self.last_query

        # use start page if the query supports it
        if query.has_macro('startPage'):
            # if the query already defined the startPage 
            # we just need to increment it
            if hasattr(query, 'startPage'):
                query.startPage += 1
            # to issue the first query startPage might not have
            # been specified, so set it to 2
            else:
                query.startPage = 2
            return query

        # otherwise the query should support startIndex
        elif query.has_macro('startIndex'):
            # if startIndex was used before we just add the 
            # items per page to it to get the next set
            if hasattr(query, 'startIndex'):
                query.startIndex += self.itemsPerPage
            # to issue the first query the startIndex may have
            # been left blank in that case we assume it to be
            # the item just after the last one on this page
            else:
                query.startIndex = self.itemsPerPage + 1
            return query

        # doesn't look like there is another stage to this query
        return None


# helper for pulling values out of a dictionary if they're there
# and returning a default value if they're not
def _pick(d,key,default=None):

    # get the value out
    value = d.get(key)
  
    # if it wasn't there return the default
    if value == None:
        return default

    # if they want an int try to convert to an int
    # and return default if it fails
    if type(default) == int:
        try:
            return int(d[key])
        except:
            return default

    # otherwise we're good to return the value
    return value

